@@ -16,12 +16,10 @@
 """Manage access to the clients, including authenticating when needed."""
 
 import logging
+import pkg_resources
+import sys
 
-from openstackclient.compute import client as compute_client
 from openstackclient.identity import client as identity_client
-from openstackclient.image import client as image_client
-from openstackclient.object import client as object_client
-from openstackclient.volume import client as volume_client
 
 
 LOG = logging.getLogger(__name__)
@@ -42,26 +40,42 @@ class ClientCache(object):
 
 class ClientManager(object):
     """Manages access to API clients, including authentication."""
-    compute = ClientCache(compute_client.make_client)
     identity = ClientCache(identity_client.make_client)
-    image = ClientCache(image_client.make_client)
-    object = ClientCache(object_client.make_client)
-    volume = ClientCache(volume_client.make_client)
 
-    def __init__(self, token=None, url=None, auth_url=None, project_name=None,
-                 project_id=None, username=None, password=None,
-                 region_name=None, api_version=None):
+    def __init__(self, token=None, url=None, auth_url=None,
+                 domain_id=None, domain_name=None,
+                 project_name=None, project_id=None,
+                 username=None, password=None,
+                 user_domain_id=None, user_domain_name=None,
+                 project_domain_id=None, project_domain_name=None,
+                 region_name=None, api_version=None, verify=True):
         self._token = token
         self._url = url
         self._auth_url = auth_url
+        self._domain_id = domain_id
+        self._domain_name = domain_name
         self._project_name = project_name
         self._project_id = project_id
         self._username = username
         self._password = password
+        self._user_domain_id = user_domain_id
+        self._user_domain_name = user_domain_name
+        self._project_domain_id = project_domain_id
+        self._project_domain_name = project_domain_name
         self._region_name = region_name
         self._api_version = api_version
         self._service_catalog = None
 
+        # verify is the Requests-compatible form
+        self._verify = verify
+        # also store in the form used by the legacy client libs
+        self._cacert = None
+        if verify is True or verify is False:
+            self._insecure = not verify
+        else:
+            self._cacert = verify
+            self._insecure = True
+
         self.auth_ref = None
 
         if not self._url:
@@ -83,3 +97,26 @@ class ClientManager(object):
             # Hope we were given the correct URL.
             endpoint = self._url
         return endpoint
+
+
+def get_extension_modules(group):
+    """Add extension clients"""
+    mod_list = []
+    for ep in pkg_resources.iter_entry_points(group):
+        LOG.debug('found extension %r' % ep.name)
+
+        __import__(ep.module_name)
+        module = sys.modules[ep.module_name]
+        mod_list.append(module)
+        init_func = getattr(module, 'Initialize', None)
+        if init_func:
+            init_func('x')
+
+        setattr(
+            ClientManager,
+            ep.name,
+            ClientCache(
+                getattr(sys.modules[ep.module_name], 'make_client', None)
+            ),
+        )
+    return mod_list
